/*
 * File:   timing.c
 * Author: Carlo Alessandro Nicolau @ INFN
 * 
 * Firmware for BPD board.
 * Timing related functions.
 * 
 */

#include "configuration.h"
#include "generated/sources/variables.h"

// number of seconds elapsed since last received command
uint32_t _timing_seconds_lc = 0;

// number of seconds elapsed since reset
uint32_t _timing_seconds_rst = 0;

// timeout in seconds before next rescue step (see rescue.c)
// initialized such as first step executed immediately
uint8_t _rescue_step_timeout = 0; 

// microsecond counter
uint32_t _tick_counter_us = 0;

// return the number of seconds of the rescue next step timeout 
uint8_t timing_get_rescue_step_timeout(void) {
    uint8_t seconds;
    INTERRUPT_GlobalInterruptDisable();
    seconds = _rescue_step_timeout;
    INTERRUPT_GlobalInterruptEnable();
    return seconds;
}

// set the number of seconds of the rescue next step timeout 
void timing_set_rescue_step_timeout(uint8_t _seconds) {    
    INTERRUPT_GlobalInterruptDisable();
    _rescue_step_timeout = _seconds;
    INTERRUPT_GlobalInterruptEnable();
}

// return the number of seconds elapsed since last received command
uint32_t timing_get_seconds_since_last_command(void) {
    uint32_t seconds;
    INTERRUPT_GlobalInterruptDisable();
    seconds = _timing_seconds_lc;
    INTERRUPT_GlobalInterruptEnable();
    return seconds;
}

// return the number of seconds elapsed since reset
uint32_t timing_get_seconds_since_reset(void) {
    uint32_t seconds;
    INTERRUPT_GlobalInterruptDisable();
    seconds = _timing_seconds_rst;
    INTERRUPT_GlobalInterruptEnable();
    return seconds;    
}

// reset the number of seconds elapsed since last received command
void timing_reset_seconds_since_last_command(void) {    
    INTERRUPT_GlobalInterruptDisable();
    _timing_seconds_lc = 0;
    INTERRUPT_GlobalInterruptEnable();
}
        
void timing_tick_irq(void) {    
    // increment microsecond counter by the number of microseconds per tick
    _tick_counter_us += IRQ_TIMER_SLOW_PERIOD_US;    
    // check if a second has elapsed
    if(_tick_counter_us >= 1000000) {        
        // ONE second elapsed!
        
        // TODO LED remove activity
        if(userpin_SWITCH_LED_D9_get()) userpin_SWITCH_LED_D9_set_low();
        else userpin_SWITCH_LED_D9_set_high();
        
        
        // increment number of seconds since last received command
        _timing_seconds_lc += 1;
        // increment number of seconds since reset
        _timing_seconds_rst += 1;
        // decrement the rescue step timeout if not zero
        if(_rescue_step_timeout > 0) {
            _rescue_step_timeout -= 1;
        }
        // decrement the microsecond counter by one second
        _tick_counter_us -= 1000000;
    }    
}